<?php

namespace FM\Service;

use FM\Factory\DashboardFacilitiesFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use FM\Service\FuncionariosHierarqService;

class DashboardFacilitiesService{

    protected $DashboardFacilitiesFactory;
    protected $Environment;
    protected $FuncionariosHierarqService;
    protected $newCode;
    protected $connection;

    public function __construct(DashboardFacilitiesFactory $DashboardFacilitiesFactory, Environment $Environment, FuncionariosHierarqService $FuncionariosHierarqService, NewCode $newCode, EntityManager $entityManager)
    {
        $this->DashboardFacilitiesFactory     = $DashboardFacilitiesFactory;
        $this->Environment                    = $Environment;
        $this->FuncionariosHierarqService     = $FuncionariosHierarqService;
        $this->newCode                        = $newCode;
        $this->connection                     = $entityManager->getConnection();
    }

    public function getDadosLimpezaFinalizada($filtro)
    {
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($this->Environment->getCdOperador());
        $params = $this->montarFiltroData($filtro);
        $filtroTipoMontado = '';
        
        $filtroTipo = array_slice($filtro, 2, 1, true);
        if ($filtroTipo && key($filtroTipo) !== 'page') {
            $filtroTipoMontado = $this->montarFiltroPorTipoTarefas($filtroTipo);
        }
        return $this->DashboardFacilitiesFactory->getDadosLimpezaFinalizada($params, $listaOperadores, $filtroTipoMontado);
    }

    public function getDadosTarefaAtrasada($filtro)
    {
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($this->Environment->getCdOperador());
        $params = $this->montarFiltroData($filtro);
        $filtroTipoMontado = '';
        
        $filtroTipo = array_slice($filtro, 2, 1, true);
        if ($filtroTipo && key($filtroTipo) !== 'page') {
            $filtroTipoMontado = $this->montarFiltroPorTipoTarefas($filtroTipo);
        }
        return $this->DashboardFacilitiesFactory->getDadosTarefaAtrasada($params, $listaOperadores, $filtroTipoMontado);
    }

    public function getDadosTarefaConcluida($filtro)
    {
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($this->Environment->getCdOperador());
        $params = $this->montarFiltroData($filtro);
        $filtroTipoMontado = '';
        
        $filtroTipo = array_slice($filtro, 2, 1, true);
        if ($filtroTipo && key($filtroTipo) !== 'page') {
            $filtroTipoMontado = $this->montarFiltroPorTipoTarefas($filtroTipo);
        }
        return $this->DashboardFacilitiesFactory->getDadosTarefaConcluida($params, $listaOperadores, $filtroTipoMontado);
    }

    public function getDadosTarefaEmAndamento($filtro)
    {
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($this->Environment->getCdOperador());
        $params = $this->montarFiltroData($filtro);
        $filtroTipoMontado = '';
        
        $filtroTipo = array_slice($filtro, 2, 1, true);
        if ($filtroTipo && key($filtroTipo) !== 'page') {
            $filtroTipoMontado = $this->montarFiltroPorTipoTarefas($filtroTipo);
        }
        return $this->DashboardFacilitiesFactory->getDadosTarefaEmAndamento($params, $listaOperadores, $filtroTipoMontado);
    }

    public function getDadosTarefaPendente($filtro)
    {
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($this->Environment->getCdOperador());
        $params = $this->montarFiltroData($filtro);
        $filtroTipoMontado = '';
        
        $filtroTipo = array_slice($filtro, 2, 1, true);
        if ($filtroTipo && key($filtroTipo) !== 'page') {
            $filtroTipoMontado = $this->montarFiltroPorTipoTarefas($filtroTipo);
        }
        return $this->DashboardFacilitiesFactory->getDadosTarefaPendente($params, $listaOperadores, $filtroTipoMontado);
    }

    public function getDadosTarefaCancelado($filtro)
    {
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($this->Environment->getCdOperador());
        $params = $this->montarFiltroData($filtro);
        $filtroTipoMontado = '';
        
        $filtroTipo = array_slice($filtro, 2, 1, true);
        if ($filtroTipo && key($filtroTipo) !== 'page') {
            $filtroTipoMontado = $this->montarFiltroPorTipoTarefas($filtroTipo);
        }
        return $this->DashboardFacilitiesFactory->getDadosTarefaCancelado($params, $listaOperadores, $filtroTipoMontado);
    }

    public function getDadosTarefaPendenteAprovacao($filtro)
    {
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($this->Environment->getCdOperador());
        $params = $this->montarFiltroData($filtro);
        $filtroTipoMontado = '';
        
        $filtroTipo = array_slice($filtro, 2, 1, true);
        if ($filtroTipo && key($filtroTipo) !== 'page') {
            $filtroTipoMontado = $this->montarFiltroPorTipoTarefas($filtroTipo);
        }
        return $this->DashboardFacilitiesFactory->getDadosTarefaPendenteAprovacao($params, $listaOperadores, $filtroTipoMontado);
    }

    public function getNaoConformidadesPorLocal($filtro)
    {
        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);
        $params = $this->montarFiltroData($filtro);

        return $this->DashboardFacilitiesFactory->getNaoConformidadesPorLocal($params, $listaOperadores);
    }

    public function getNaoConformidades($filtro) 
    {
        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);
        $params = $this->montarFiltroData($filtro);

        return $this->DashboardFacilitiesFactory->getNaoConformidades($params, $listaOperadores); 
    }

    public function getTarefasPorCliente($filtro)
    {
        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);
        $params = $this->montarFiltroData($filtro);

        return $this->DashboardFacilitiesFactory->getTarefasPorCliente($params, $listaOperadores); 
    }

    public function getTarefasPorLocal($filtro)
    {
        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);
        $params = $this->montarFiltroData($filtro);

        return $this->DashboardFacilitiesFactory->getTarefasPorLocal($params, $listaOperadores);
    }

    
    public function getTarefasPorChecklist($filtro)
    {
        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);
        $params = $this->montarFiltroData($filtro);

        return $this->DashboardFacilitiesFactory->getTarefasPorChecklist($params, $listaOperadores);
    }

    public function getVistoriasPendentes($filtro)
    {
        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);
        $params = $this->montarFiltroData($filtro);
        $filtroTipoMontado = '';

        $filtroTipo = array_slice($filtro, 2, 1, true);
        if($filtroTipo && key($filtroTipo) !== 'page') {
            $filtroTipoMontado = $this->montarFiltroPorTipoVistorias($filtroTipo);
        }
        return $this->DashboardFacilitiesFactory->getVistoriasPendentes($params, $listaOperadores, $filtroTipoMontado);
    }
    
    public function getVistoriasAtrasadas($filtro)
    {
        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);
        $params = $this->montarFiltroData($filtro);
        $filtroTipoMontado = '';

        $filtroTipo = array_slice($filtro, 2, 1, true);
        if($filtroTipo && key($filtroTipo) !== 'page') {
            $filtroTipoMontado = $this->montarFiltroPorTipoVistorias($filtroTipo);
        }
        return $this->DashboardFacilitiesFactory->getVistoriasAtrasadas($params, $listaOperadores, $filtroTipoMontado);
    }

    public function getVistoriasAvaliadas($filtro)
    {
        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);
        $params = $this->montarFiltroData($filtro);
        $filtroTipoMontado = '';

        $filtroTipo = array_slice($filtro, 2, 1, true);
        if($filtroTipo && key($filtroTipo) !== 'page') {
            $filtroTipoMontado = $this->montarFiltroPorTipoVistorias($filtroTipo);
        }
        return $this->DashboardFacilitiesFactory->getVistoriasAvaliadas($params, $listaOperadores, $filtroTipoMontado);
    }

    public function getVistoriasEmAndamento($filtro)
    {
        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);
        $params = $this->montarFiltroData($filtro);
        $filtroTipoMontado = '';

        $filtroTipo = array_slice($filtro, 2, 1, true);
        if($filtroTipo && key($filtroTipo) !== 'page') {
            $filtroTipoMontado = $this->montarFiltroPorTipoVistorias($filtroTipo);
        }
        return $this->DashboardFacilitiesFactory->getVistoriasEmAndamento($params, $listaOperadores, $filtroTipoMontado);
    }

    public function getVistoriasCancelado($filtro)
    {
        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);
        $params = $this->montarFiltroData($filtro);
        $filtroTipoMontado = '';

        $filtroTipo = array_slice($filtro, 2, 1, true);
        if($filtroTipo && key($filtroTipo) !== 'page') {
            $filtroTipoMontado = $this->montarFiltroPorTipoVistorias($filtroTipo);
        }
        return $this->DashboardFacilitiesFactory->getVistoriasCancelado($params, $listaOperadores, $filtroTipoMontado);
    }

    public function getVistoriasPorCliente($filtro)
    {
        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);
        $params = $this->montarFiltroData($filtro);

        return $this->DashboardFacilitiesFactory->getVistoriasPorCliente($params, $listaOperadores);
    }

    public function getVistoriasPorChecklist($filtro)
    {
        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);
        $params = $this->montarFiltroData($filtro);

        return $this->DashboardFacilitiesFactory->getVistoriasPorChecklist($params, $listaOperadores);
    }

    public function getVistoriasPorLocal($filtro)
    {
        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);
        $params = $this->montarFiltroData($filtro);

        return $this->DashboardFacilitiesFactory->getVistoriasPorLocal($params, $listaOperadores);
    }

    public function getAuditoriasConcluidas($filtro)
    {
        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);
        $params = $this->montarFiltroData($filtro);
        $filtroTipoMontado = '';

        $filtroTipo = array_slice($filtro, 2, 1, true);
        if($filtroTipo && key($filtroTipo) !== 'page') {
            $filtroTipoMontado = $this->montarFiltroPorTipoAuditoria($filtroTipo);
        }
        return $this->DashboardFacilitiesFactory->getAuditoriasConcluidas($params, $listaOperadores, $filtroTipoMontado);
    }

    public function getAuditoriasEmAndamento($filtro)
    {
        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);
        $params = $this->montarFiltroData($filtro);
        $filtroTipoMontado = '';

        $filtroTipo = array_slice($filtro, 2, 1, true);
        if($filtroTipo && key($filtroTipo) !== 'page') {
            $filtroTipoMontado = $this->montarFiltroPorTipoAuditoria($filtroTipo);
        }
        return $this->DashboardFacilitiesFactory->getAuditoriasEmAndamento($params, $listaOperadores, $filtroTipoMontado);
    }

    public function getAuditoriasAtrasadas($filtro)
    {
        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);
        $params = $this->montarFiltroData($filtro);
        $filtroTipoMontado = '';

        $filtroTipo = array_slice($filtro, 2, 1, true);
        if($filtroTipo && key($filtroTipo) !== 'page') {
            $filtroTipoMontado = $this->montarFiltroPorTipoAuditoria($filtroTipo);
        }
        return $this->DashboardFacilitiesFactory->getAuditoriasAtrasadas($params, $listaOperadores, $filtroTipoMontado);
    }

    public function getAuditoriasPendentes($filtro)
    {
        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);
        $params = $this->montarFiltroData($filtro);
        $filtroTipoMontado = '';

        $filtroTipo = array_slice($filtro, 2, 1, true);
        if($filtroTipo && key($filtroTipo) !== 'page') {
            $filtroTipoMontado = $this->montarFiltroPorTipoAuditoria($filtroTipo);
        }
        return $this->DashboardFacilitiesFactory->getAuditoriasPendentes($params, $listaOperadores, $filtroTipoMontado);
    }

    public function getAuditoriasPorLocal($filtro)
    {
        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);
        $params = $this->montarFiltroData($filtro);

        return $this->DashboardFacilitiesFactory->getAuditoriasPorLocal($params, $listaOperadores,);
    }

    public function getAuditoriasPorCliente($filtro)
    {
        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);
        $params = $this->montarFiltroData($filtro);

        return $this->DashboardFacilitiesFactory->getAuditoriasPorCliente($params, $listaOperadores,);
    }

    public function getAuditoriasPorModelo($filtro)
    {
        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);
        $params = $this->montarFiltroData($filtro);

        return $this->DashboardFacilitiesFactory->getAuditoriasPorModelo($params, $listaOperadores,);
    }

    public function getOrdens($filtro)
    {
        $params = $this->montarFiltroData($filtro);

        return $this->DashboardFacilitiesFactory->getOrdens($params);
    }

    public function getDadosOrdens($filtro)
    {
        $params = $this->montarFiltroData($filtro);
        $filtroTipo = $this->montarFiltroTipoOrdens($filtro);

        return $this->DashboardFacilitiesFactory->getDadosOrdens($params, $filtroTipo);
    }

    public function getOrdensEquipamento($filtro)
    {
        $params = $this->montarFiltroData($filtro);

        return $this->DashboardFacilitiesFactory->getOrdensEquipamento($params);
    }

    public function getOrdensLocal($filtro)
    {
        $params = $this->montarFiltroData($filtro);

        return $this->DashboardFacilitiesFactory->getOrdensLocal($params);
    }

    public function getSolicitacoes($filtro)
    {
        $params = $this->montarFiltroData($filtro);

        return $this->DashboardFacilitiesFactory->getSolicitacoes($params);
    }

    public function getDadosSolicitacoes($filtro)
    {
        $params = $this->montarFiltroData($filtro);
        $filtroTipo = $this->montarFiltroTipoSolicitacoes($filtro);

        return $this->DashboardFacilitiesFactory->getDadosSolicitacoes($params, $filtroTipo);
    }

    private function montarFiltroPorTipoTarefas($param) 
    {
        $filtro = '';
        foreach ($param as $chave => $valor) {
            if($chave === 'CDCLIENTE') {
                $filtro .= " AND CLIENTE." . $chave . " = '" . $valor . "'";
            } else {
                $filtro .= " AND FCM_CHECKLISTAGEND." . $chave . " = '" . $valor . "'";
            }
        }

       return$filtro;
    }

    private function montarFiltroPorTipoVistorias($param) 
    {
        $filtro = '';
        foreach ($param as $chave => $valor) {
            if($chave === 'CDCLIENTE') {
                $filtro .= " AND CLIENTE." . $chave . " = '" . $valor . "'";
            } else if($chave === 'NRCHECKLIST'){
                $filtro .= " AND FCM_CHECKLIST." . $chave . " = '" . $valor . "'";
            } else {
                $filtro .= " AND FCM_LOCALCLI." . $chave . " = '" . $valor . "'";
            }
        }

       return$filtro;
    }

    private function montarFiltroPorTipoAuditoria($param) 
    {
        $filtro = '';
        foreach ($param as $chave => $valor) {
            if($chave === 'CDCLIENTE') {
                $filtro .= " AND CLIENTE." . $chave . " = '" . $valor . "'";
            } else if($chave === 'NRMODELOAUDIT'){
                $filtro .= " AND FCM_MODELOAUDIT." . $chave . " = '" . $valor . "'";
            } else {
                $filtro .= " AND FCM_LOCALCLI." . $chave . " = '" . $valor . "'";
            }
        }

       return$filtro;
    }

    private function montarFiltroTipoSolicitacoes($param)
    {
        $filtro = "";
        foreach ($param as $chave => $valor) {
            if($chave === "SITUACAO") {
                if($valor === "Recusadas") {
                    $filtro .= "AND SOLSERV.IDSITUACAO = '5'";
                }
                if($valor === "Abertas") {
                    $filtro .= "AND SOLSERV.IDSITUACAO = '1'";
                }
            }
        }
        return $filtro;
    }

    private function montarFiltroTipoOrdens($param)
    {
        $filtro = '';
        foreach ($param as $chave => $valor) {
            if($chave === 'SITUACAO') {
                if($valor === 'Pausadas') {
                    $filtro .= "AND OSE.IDSITUACAO = '5'";
                }
                if($valor === 'Em Andamento') {
                    $filtro .= "AND OSE.IDSITUACAO = '2'";
                }
                if($valor === 'Finalizadas') {
                    $filtro .= "AND OSE.IDSITUACAO = '3'";
                }
                if($valor === 'Pendentes') {
                    $filtro .= "AND OSE.IDSITUACAO = '1'";
                }
                if($valor === 'Recusadas') {
                    $filtro .= "AND OSE.IDSITUACAO = '4'";
                }
            }
            if($chave === "NRLOCALCLIENTE") {
                $filtro .= " AND OSE.". $chave . " = '". $valor . "'";
            }
        }
        return $filtro;
    }

    private function montarFiltroData($filtro)
    {
        if (!empty($filtro['DTRANGEINICIO'])) {
            return [
                "CDOPERADOR" => $this->Environment->getCdOperador(),
                "DTRANGEINICIO" => $filtro['DTRANGEINICIO'],
                "DTRANGEFINAL" => $filtro['DTRANGEFINAL'],
            ];
        }
        return ["CDOPERADOR" => $this->Environment->getCdOperador()];
    }
}