<?php

namespace FM\Service;

use FM\Factory\Fm0000000EquipesFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use FM\Service\FindService;

class Fm0000000EquipesService
{

    protected $EquipesFactory;
    protected $Environment;
    protected $newCode;
    protected $connection;
    protected $FindService;

    public function __construct(Fm0000000EquipesFactory $Fm0000000EquipesFactory, Environment $Environment, NewCode $newCode, EntityManager $entityManager, FindService $FindService)
    {
        $this->EquipesFactory                 = $Fm0000000EquipesFactory;
        $this->Environment                    = $Environment;
        $this->newCode                        = $newCode;
        $this->connection                     = $entityManager->getConnection();
        $this->FindService                    = $FindService;
    }
    // FUNÇÕES DO FRAME PRINCIPAL EQUIPES
    public function getEquipes()
    {
        return $this->EquipesFactory->getEquipes();
    }

    public function salvarEquipes($params)
    {
        $this->connection->beginTransaction();
        try {
            $nrorg =  $this->Environment->getNrOrg();
            $operador = $this->Environment->getCdOperador();
            $tarefasAssociadas = [];
            
            foreach ($params as $row) {
                if ($row["NREQUIPE"] == 0) {
                    $param = array(
                        "NREQUIPE"          => $this->newCode->getCodeByTableName("FCM_EQUIPE", $nrorg, 1, 10, true),
                        "NMEQUIPE"          => $row["NMEQUIPE"],
                        "IDATIVO"           => $row["IDATIVO"],
                        "NRORG"             => $nrorg,
                        "NRORGINCLUSAO"     => $nrorg,
                        "CDOPERINCLUSAO"    => $operador
                    );

                    $this->EquipesFactory->inserirEquipe($param);
                } else if($row["IDATIVO"] === 'S') {
                    $this->atualizarEquipe($row);
                } else {
                    $tarefasEquipe = $this->buscarTarefaEquipe(array(
                        "NREQUIPE"  => $row['NREQUIPE']
                    ));

                    $tarefasFiltradas['data'] = array_values(array_filter($tarefasEquipe['data'], function ($item) {
                        return !in_array($item['IDSITUACAOAGEND'], ['3', '4']);
                    }));

                    if (empty($tarefasFiltradas)) {
                        $this->atualizarEquipe($row);
                    } else {
                        $tarefasAssociadas = $this->preencheArrayTarefasAssociadas($tarefasFiltradas, $tarefasAssociadas);
                    }

                }
            }

            $this->connection->commit();
            return $tarefasAssociadas;
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function atualizarEquipe($row)
    {
        $this->connection->beginTransaction();
        try {
            $nrorg =  $this->Environment->getNrOrg();
            $operador = $this->Environment->getCdOperador();

            $param = array(
                "NREQUIPE"          => $row["NREQUIPE"],
                "NMEQUIPE"          => $row["NMEQUIPE"],
                "IDATIVO"           => $row["IDATIVO"],
                "CDOPERULTATU"      => $operador,
                "NRORGULTATU"       => $nrorg
            );
            $this->EquipesFactory->atualizarEquipe($param);

        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getOperadorEquipe($params)
    {
        $param = array(
            "NREQUIPE"    => $params["EQUIPE"],
        );
        return $this->EquipesFactory->getOperadorEquipe($param);
    }

    public function relacionarOperadorEquipe($params)
    {
        $this->connection->beginTransaction();
        try {
            $nrorg =  $this->Environment->getNrOrg();
            $operador = $this->Environment->getCdOperador();

            $param = array(
                "NRRELACEQUIPEOPER"    => $this->newCode->getCodeByTableName("FCM_RELACEQUIPEOPER", $nrorg, 1, 10, true),
                "NREQUIPE"          => $params["NREQUIPE"],
                "CDOPERADOR"        => $params["OPERADOR"],
                "NRORG"             => $nrorg,
                "NRORGINCLUSAO"     => $nrorg,
                "CDOPERINCLUSAO"    => $operador,
                "IDATIVO"           => $params["IDATIVO"],
            );

            $this->EquipesFactory->relacionarOperadorEquipe($param);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function preencheArrayTarefasAssociadas($tarefasEquipe, $tarefasAssociadas)
    {
        // Preenche o array de tarefas associadas sem repetição de tarefas
        foreach ($tarefasEquipe['data'] as $value) {
            // Verifica diretamente dentro de $tarefasAssociadas
            $jaExiste = false;

            foreach ($tarefasAssociadas as $tarefas) {
                if ($tarefas['NRCHECKLISTAGEND'] === $value['NRCHECKLISTAGEND']) {
                    $jaExiste = true;
                    break;
                }
            }

            // Insere a taarefa apenas se não for duplicada
            if (!$jaExiste) {
                $tarefasAssociadas[] = [
                    'NRCHECKLISTAGEND'  => $value['NRCHECKLISTAGEND'],
                    'NMCHECKLISTAGEND'  => $value['NMCHECKLISTAGEND'],
                    'DTHORAAGEND'       => $value['DTHORAAGEND'],
                    'NMFANTCLIE'        => $value['NMFANTCLIE'],
                    'NMLOCALCLIENTE'    => $value['NMLOCALCLIENTE'],
                    'IDSITUACAOAGEND'   => $value['IDSITUACAOAGEND']
                ];
            }
        }

        return $tarefasAssociadas;
    }
    public function excluirEquipes($params)
    {
        $this->connection->beginTransaction();
        try {
            $equipesAExcluir = array();
            $tarefasAssociadas = [];
            $filtro = '';
            
            foreach ($params as $nr) {
                $tarefasEquipe = $this->buscarTarefaEquipe(array(
                    "NREQUIPE"  => $nr
                ));

                if (empty($tarefasEquipe['data'])) {
                    array_push($equipesAExcluir, $nr);
                } else {
                    $tarefasAssociadas = $this->preencheArrayTarefasAssociadas($tarefasEquipe, $tarefasAssociadas);
                }
            }

            
            if (!empty($equipesAExcluir)) {
                $nrFormatado = $this->FindService->formatarArray($equipesAExcluir);
                $filtro = "WHERE NREQUIPE IN (".$nrFormatado.")";

                $this->EquipesFactory->excluirOperadorEquipe($filtro);
                $this->EquipesFactory->excluirEquipes($nrFormatado);
            }

            $this->connection->commit();
            return $tarefasAssociadas;
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }    

    public function excluirOperadorEquipe($params)
    {
        $this->connection->beginTransaction();
        try {
            $filtro = '';
            $nrFormatado = $this->FindService->formatarArray($params);

            $filtro = "WHERE NRRELACEQUIPEOPER IN (".$nrFormatado.")";
            $this->EquipesFactory->excluirOperadorEquipe($filtro);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function buscarTarefaEquipe($nrEquipe) 
    {
        try{
            return $this->EquipesFactory->buscarTarefaEquipe($nrEquipe);
        } catch (\Exception $e) {
            throw $e;
        }
    }
}
