import { Utilities } from '@/util/Utilities';
import {
    CheckboxMultiple, Form, LoadingService, Modal, ModalService, Text,
    Textarea,
} from '@zeedhi/common';
import {
    Http, IDictionary, IEventParam, Metadata, SimpleDatasource,
} from '@zeedhi/core';
import { IterableCarousel, TekGrid } from '@zeedhi/teknisa-components-common';
import { IKanbanEventParam, Kanban, KanbanTask } from '@zeedhi/zd-kanban-common';

export class FmKanbanTarefas {
    public modal!: Modal;

    public modalMoved!: Modal;

    public nrChecklistAgend: string = '';

    public visibilidadeColunaAtraso({ component }: any) {
        const situacao = component.name.split('_')[1];
        const atraso = component.name.split('_')[2];

        if (situacao === '1' && atraso !== '  ') {
            component.isVisible = true;
        } else {
            component.isVisible = false;
        }
    }

    public async criarModais() {
        this.modal = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00004_000_tarefas/modal/visualizar_infos_kanban_tarefas' });
        this.modalMoved = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00004_000_tarefas/modal/fm_modal_mover_task' });
    }

    public destruirModais() {
        this.modal.destroy();
        this.modalMoved.destroy();
    }

    public async abrirModal({ component }: IEventParam<KanbanTask>) {
        /* eslint-disable */
        this.nrChecklistAgend = component.name.split('_')[0];
        const idsituacao = component.name.split('_')[1];
        /* eslint-enable */
        if (idsituacao !== '1') {
            this.modal.show();
            this.montarInformacoesComplementares();
            this.carregarFotos();
        }
    }

    public fecharModal() {
        this.modal.hide();
    }

    public async montarInformacoesComplementares() {
        const nome = Metadata.getInstance<Text>('id-tarefa');
        const checkbox = Metadata.getInstance<CheckboxMultiple>('checkbox-itens');
        checkbox.value = [];

        const observacoes = Metadata.getInstance<Textarea>('observacoes-tarefas');
        const semItens = Metadata.getInstance<Text>('nao-ha-itens');
        let mensagem = '';

        LoadingService.show();
        Http.post('/getItensTarefasKanban', {
            NRCHECKLISTAGEND: this.nrChecklistAgend,
        }).then((response: IDictionary) => {
            const dados = response.data.data;
            nome.text = dados[0].NMCHECKLISTAGEND;

            checkbox.value = dados.filter((elemento: any) => elemento.IDREALIZADO === 'S').map((elemento: any) => elemento.NMTAREFA);
            (checkbox.datasource as SimpleDatasource)!.items = dados.map((elemento: any) => elemento.NMTAREFA);

            if (dados.length > 0) {
                semItens.isVisible = false;
            } else {
                semItens.isVisible = true;
            }

            if (dados[0].DSJUSTCANCELAMENTO !== null && dados[0].DSOBSTAREFAAGEND !== null) {
                mensagem = `Justificativa de cancelamento: ${dados[0].DSJUSTCANCELAMENTO} - Observações da Tarefa: ${dados[0].DSOBSTAREFAAGEND}`;
            } else if (dados[0].DSJUSTCANCELAMENTO !== null) {
                mensagem = `Justificativa de cancelamento: ${dados[0].DSJUSTCANCELAMENTO}`;
            } else if (dados[0].DSOBSTAREFAAGEND !== null) {
                mensagem = `Observações da Tarefa: ${dados[0].DSOBSTAREFAAGEND}`;
            } else {
                mensagem = 'Não há observações';
            }
            observacoes.value = mensagem;
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public carregarFotos() {
        const carousel = Metadata.getInstance<IterableCarousel>('carousel-imagens-anexo-vistorias');
        carousel.iterable.datasource.data = [];

        LoadingService.show();
        Http.post('/getFotosTarefas', {
            NRCHECKLISTAGEND: this.nrChecklistAgend,
        }).then((response: any) => {
            if (response.data.data.length === 0) {
                carousel.isVisible = false;
                Metadata.getInstance<Text>('sem-resultado-imagens-tarefas').isVisible = true;
            } else {
                response.data.data.forEach((dados: IDictionary) => {
                    carousel.iterable.datasource.data.push({
                        NRFOTOTAREFAAGEND: dados.NRFOTOTAREFAAGEND,
                        DSFOTOTAREFAAGEND: dados.DSFOTOTAREFAAGEND,
                    });
                });
            }
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public task: any;

    public openModalMoveTask({ task }: IKanbanEventParam) {
        if (task.columnFrom.id !== task.columnTo.id) {
            this.task = task;
            this.modalMoved.show();
            Metadata.getInstance<Form>('form-justificativa-moved-task').clearForm();
        }
    }

    public closeModalMoveTask() {
        const kanban = Metadata.getInstance<Kanban>('kanban-tarefas');
        kanban.moveTask(this.task.task, { index: this.task.oldIndex, columnId: this.task.columnFrom.id });
        this.modalMoved.hide();
    }

    public moveTask() {
        const form = Metadata.getInstance<Form>('form-justificativa-moved-task');
        const gridPrincipal = Metadata.getInstance<TekGrid>('tek-grid-tarefas');

        if (Utilities.validateForm(form)) {
            LoadingService.show();
            Http.post('/movedTask', {
                IDSITUACAOAGEND: this.task.columnTo.id,
                NRCHECKLISTAGEND: this.task.task.NRCHECKLISTAGEND,
                NMCHECKLISTAGEND: this.task.task.NMCHECKLISTAGEND,
                JUSTIFICATIVA: Metadata.getInstance<Textarea>('text-area-justificativa-moved-task').value,
            }).then((response: IDictionary) => {
                Utilities.showSuccessMessage();
                this.closeModalMoveTask();
                gridPrincipal.datasource.get();
                if (response.data) {
                    Utilities.alertMessage('Houve uma tentativa de reportar essa Tarefa. Porém, a mesma não possui Vistoria e por isso foi cancelada sem necessidade de aprovação prévia', 'warning');
                }
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }
}
