import {
    Http, IDictionary, Metadata, SimpleDatasource,
} from '@zeedhi/core';
import {
    CheckboxMultiple, LoadingService, Modal, ModalService, Select, SelectMultiple, Text, Textarea, Grid,
    TextInput,
    Icon,
} from '@zeedhi/common';
import { Utilities } from '@/util/Utilities';
import { IterableCarousel } from '@zeedhi/teknisa-components-common';
import { RichText } from '@zeedhi/zd-richtext-common';

export class RelatorioTarefasController {
    private modal!: Modal;

    public currentRow: any;

    public currentModeloPop: any;

    public async criarModais() {
        this.modal = await ModalService.createFromJson('fm_00028_000_relatorios/modal/fm_modal_info_relatorio_tarefas', true);
    }

    public async destruirModais() {
        this.modal.destroy();
    }

    public abrirModal({ row }: any) {
        this.modal.show();
        this.currentRow = row;
        setTimeout(() => {
            this.montarInformacoesComplementares();
        }, 70);
    }

    public closeModal() {
        const grid = Metadata.getInstance<Grid>('grid-historico-tarefas');
        grid.datasource.updateData([]);
        this.modal.hide();
    }

    public setFiltro({ component }: any) {
        component.datasource.dynamicFilter = {
            IDATIVO: [{
                relation: 'AND',
                operation: 'IN',
                value: 'S',
            }],
        };
        component.datasource.get();
    }

    public montarInformacoesComplementares() {
        LoadingService.show();
        const nr = this.currentRow.NRCHECKLISTAGEND;
        const checkbox = Metadata.getInstance<CheckboxMultiple>('checkbox-itens-tarefas');
        const observacoes = Metadata.getInstance<Textarea>('observacoes-tarefas');
        const semItens = Metadata.getInstance<Text>('nao-ha-itens');
        const select = Metadata.getInstance<SelectMultiple>('select-epi-info');
        const carousel = Metadata.getInstance<IterableCarousel>('carousel-imagens-anexo-tarefas');
        let mensagem = '';
        this.loadModeloPopTarefa();

        select.datasource.setFilter({
            NRCHECKLISTAGEND: nr,
        });

        Http.post('/getItensTarefas', {
            NRCHECKLISTAGEND: nr,
        }).then((response: IDictionary) => {
            const dados = response.data.data;

            select.setValue(select.datasource.data);

            checkbox.value = dados.filter((elemento: any) => elemento.IDREALIZADO === 'S').map((elemento: any) => elemento.NMTAREFA);
            (checkbox.datasource as SimpleDatasource)!.items = dados.map((elemento: any) => elemento.NMTAREFA);

            if (dados.length > 0) {
                semItens.isVisible = false;
            } else {
                semItens.isVisible = true;
            }

            if (this.currentRow.DSJUSTCANCELAMENTO !== null && this.currentRow.DSOBSTAREFAAGEND !== null) {
                mensagem = `Justificativa de cancelamento: ${this.currentRow.DSJUSTCANCELAMENTO} - Observações da Tarefa: ${this.currentRow.DSOBSTAREFAAGEND}`;
            } else if (this.currentRow.DSJUSTCANCELAMENTO !== null) {
                mensagem = `Justificativa de cancelamento: ${this.currentRow.DSJUSTCANCELAMENTO}`;
            } else if (this.currentRow.DSOBSTAREFAAGEND !== null) {
                mensagem = `Observações da Tarefa: ${this.currentRow.DSOBSTAREFAAGEND}`;
            } else {
                mensagem = 'Não há observações';
            }
            observacoes.value = mensagem;
            this.carregarFotos({ component: carousel });
            this.loadHistoricoTarefa();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public carregarFotos({ component }: any) {
        const nr = this.currentRow.NRCHECKLISTAGEND;

        LoadingService.show();
        Http.post('/getFotosTarefas', {
            NRCHECKLISTAGEND: nr,
        }).then((response: any) => {
            if (response.data.data.length === 0) {
                component.isVisible = false;
                Metadata.getInstance<Text>('sem-resultado-imagens-tarefas').isVisible = true;
            } else {
                response.data.data.forEach((dados: IDictionary) => {
                    component.iterable.datasource.data.push({
                        NRFOTOTAREFAAGEND: dados.NRFOTOTAREFAAGEND,
                        DSFOTOTAREFAAGEND: dados.DSFOTOTAREFAAGEND,
                    });
                });
            }
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public rowStyleConditions(row: any) {
        if (row.ATRASO !== null) {
            if (row.ATRASO.split(' ')[1] === 'dias') {
                if (row.ATRASO.split(' ')[0] >= 1) {
                    return { cssClass: 'linha-grid-verificada' };
                }
            }
        }

        return { cssClass: '' };
    }

    public loadHistoricoTarefa() {
        const grid = Metadata.getInstance<Grid>('grid-historico-tarefas');
        grid.datasource.setFilter({
            NRCHECKLISTAGEND: this.currentRow.NRCHECKLISTAGEND,
        });
    }

    public loadModeloPopTarefa() {
        const selectModeloPop = Metadata.getInstance<Select>('select-modelopop-info');
        const descricaoModeloPop = Metadata.getInstance<RichText>('descricao-modelo-pop');
        const linkModeloPop = Metadata.getInstance<TextInput>('url-modelo-pop');
        const btnBaixar = Metadata.getInstance<Icon>('download-modelo-pop');

        Http.get(`/getModelosPopTarefa?NRCHECKLISTAGEND=${this.currentRow.NRCHECKLISTAGEND}`).then(async (response) => {
            const [modeloPop = {}] = response.data?.data ?? [];

            selectModeloPop.updateItems([modeloPop.NMMODELOPOP]);
            selectModeloPop.setValue(modeloPop.NMMODELOPOP);
            descricaoModeloPop.value = modeloPop.DESCMODELOPOP || '<p> Não há descrição </p>';
            linkModeloPop.value = modeloPop.LINKMODELOPOP || '';
            this.currentModeloPop = modeloPop;

            btnBaixar.disabled = !modeloPop.NMARQUIVO;
        });
    }

    public async downloadAnexoModeloPop() {
        try {
            if (!this.currentModeloPop.NMARQUIVO) {
                Utilities.showErrorMessage('Esse modelo de procedimento não tem arquivos');
                return;
            }

            const anexo = await fetch(`/api/getFile?fileName=${encodeURIComponent(this.currentModeloPop.NMARQUIVO)}`);

            if (!anexo.ok) {
                throw new Error(`Erro ao baixar o arquivo: ${anexo.statusText}`);
            }

            const fileBlob = await anexo.blob();
            const url = URL.createObjectURL(fileBlob);

            const a = document.createElement('a');
            a.href = url;
            a.download = this.currentModeloPop.NMARQUIVO || 'arquivo-desconhecido';
            document.body.appendChild(a);
            a.click();

            document.body.removeChild(a);
            URL.revokeObjectURL(url);
        } catch (error) {
            Utilities.showErrorMessage(`Erro ao baixar o arquivo: ${error}`);
        }
    }
}
