import { Utilities } from '@/util/Utilities';
import {
    Form, IterableComponentRender, LoadingService, Modal, ModalService, Textarea,
} from '@zeedhi/common';
import {
    Http, IEventParam, Metadata,
} from '@zeedhi/core';
import { IterableCarousel, TekGrid } from '@zeedhi/teknisa-components-common';
import { TekIterableComponentRender } from '@zeedhi/teknisa-components-vuetify';
import { IKanbanEventParam, Kanban } from '@zeedhi/zd-kanban-common';

export class FmKanbanAuditoriaController {
    public modal!: Modal;

    public nrauditoria: string = '';

    public modalMoved!: Modal;

    public async abrirModal({ component }: any) {
        /* eslint-disable */
        this.nrauditoria = component.name.split('_')[0];
        const idsituacao = component.name.split('_')[1];
        /* eslint-enable */

        if (idsituacao !== '1') {
            await this.modal.show();
            this.onMountedCategorias();
        }
    }

    public async criarModais() {
        this.modal = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00040_000_auditoria/modal/visualizar_infos_kanban_auditorias' });
        this.modalMoved = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00040_000_auditoria/modal/fm_modal_mover_task' });
    }

    public destruirModais() {
        this.modal.destroy();
        this.modalMoved.destroy();
    }

    public fecharModal() {
        this.modal.hide();
    }

    public onMountedCategorias() {
        Metadata.getInstance<TekIterableComponentRender>('iterable-categorias-kanban').datasource.setFilter({
            NRAUDITORIA: this.nrauditoria,
        });
    }

    public getTopicos({ component }: IEventParam<IterableComponentRender>) {
        component.datasource.setFilter({
            NRCATEGORIAAUDIT: component.name.split('_')[1],
            NRAUDITORIA: this.nrauditoria,
        });
    }

    public getCriterios({ component }: IEventParam<TekIterableComponentRender>) {
        component.datasource.setFilter({
            NRTOPICOAUDIT: component.name.split('_')[1],
            NRCATEGORIAAUDIT: component.name.split('_')[2],
            NRAUDITORIA: this.nrauditoria,
        });
    }

    public onMountedImagensAnexadas({ component }: IEventParam<IterableCarousel>) {
        component.iterable.datasource.setFilter({
            NRAUDITORIA: this.nrauditoria,
            NRCATEGORIAAUDIT: component.name.split('_')[2],
            NRTOPICOAUDIT: component.name.split('_')[1],
        });
    }

    public task: any;

    public async openModalMoveTask({ task }: IKanbanEventParam) {
        this.task = task;
        if (task.columnFrom.id !== task.columnTo.id) {
            await this.modalMoved.show();
            Metadata.getInstance<Form>('form-justificativa-moved-task').clearForm();
        }
    }

    public closeModalMoveTask() {
        const kanban = Metadata.getInstance<Kanban>('kanban-auditoria');
        kanban.moveTask(this.task.task, { index: this.task.oldIndex, columnId: this.task.columnFrom.id });
        this.modalMoved.hide();
    }

    public moveTask() {
        const form = Metadata.getInstance<Form>('form-justificativa-moved-task');
        const gridPrincipal = Metadata.getInstance<TekGrid>('tek-grid-tarefas-auditoria');

        if (Utilities.validateForm(form)) {
            LoadingService.show();
            Http.post('/movedTaskAuditoria', {
                IDSITUACAOAUDIT: this.task.columnTo.id,
                NRAUDITORIA: this.task.task.NRAUDITORIA,
                NMAUDITORIA: this.task.task.NMAUDITORIA,
                JUSTIFICATIVA: Metadata.getInstance<Textarea>('text-area-justificativa-moved-task').value,
            }).then(() => {
                Utilities.showSuccessMessage();
                this.closeModalMoveTask();
                gridPrincipal.datasource.get();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }
}
