import { Utilities } from '@/util/Utilities';
import { Card, IterableComponentRender } from '@zeedhi/common';
import { Metadata } from '@zeedhi/core';
import { FieldGroup } from '@zeedhi/zd-field-group';

export class FmMesaOperacoesController {
    public visibilidadeCardPrincipalDashboard: boolean = true;

    public visibilidadeFrameOperadores: boolean = false;

    public currentRow: any;

    public nomeTarefaData: string = '';

    public visualizarInfoResponsaveis({ row }: any) {
        if (row.IDTAREFAEQUIPE !== 'N' || row.CDOPERADOR !== null) {
            this.visibilidadeCardPrincipalDashboard = false;
            this.visibilidadeFrameOperadores = !this.visibilidadeCardPrincipalDashboard;
            this.nomeTarefaData = `${row.NMCHECKLISTAGEND} - ${row.DTHORAAGEND}`;
            this.currentRow = row;
            this.setFiltroOperadores();
        } else {
            Utilities.alertMessage('Não é possível exibir informações para tarefas sem responsável definido.', 'warning');
        }
    }

    public retornarTelaDashboard() {
        this.visibilidadeCardPrincipalDashboard = true;
        this.visibilidadeFrameOperadores = !this.visibilidadeCardPrincipalDashboard;
    }

    public async setFiltroOperadores() {
        const iterable = Metadata.getInstance<IterableComponentRender>('iterable-operadores');
        iterable.datasource.data = [];

        await iterable.datasource.setFilter({
            CDOPERADOR: this.currentRow.CDOPERADOR,
            NMOPERADOR: this.currentRow.NMOPERADOR,
            IDTAREFAEQUIPE: this.currentRow.IDTAREFAEQUIPE,
            NREQUIPE: this.currentRow.NREQUIPE,
            NRCHECKLISTAGEND: this.currentRow.NRCHECKLISTAGEND,
            NRPARCNEGOCIO: this.currentRow.NRPARCNEGOCIO,
            DTHORAAGEND: this.currentRow.DTHORAAGEND.split(' ')[0],
            DTHRFINALAGEND: this.currentRow.DTHRFINALAGEND.split(' ')[0],
        });

        if (iterable.datasource.data.length === 1) {
            Metadata.getInstance<FieldGroup>(`field-group-operador_${iterable.datasource.data[0].CDOPERADOR}`).expanded = true;
        }
        this.visibilidadeCards(iterable.datasource.data);
    }

    public visibilidadeCards(data: any) {
        if (data.length > 0) {
            data.forEach((operador: any) => {
                const isNull = operador.NRINSCRICAOPARC === null;

                if (isNull) Metadata.getInstance<FieldGroup>(`field-group-operador_${operador.CDOPERADOR}`).expanded = true;
                // Apenas "card-aviso" visível se for null
                Metadata.getInstance<Card>(`card-aviso_${operador.CDOPERADOR}`).isVisible = isNull;

                // Os outros cards são visíveis apenas se NÃO for null
                Metadata.getInstance<Card>(`card-ponto_${operador.CDOPERADOR}`).isVisible = !isNull;
                Metadata.getInstance<Card>(`card-afastamento_${operador.CDOPERADOR}`).isVisible = !isNull;
                Metadata.getInstance<Card>(`card-ferias_${operador.CDOPERADOR}`).isVisible = !isNull;
            });
        }
    }
}
